unit DS004U1;
(*Demonstration of access to counter in DS2324.

(*
*)

(*

Terminology: MicroLan, iButton and 1-Wire are all trademarks of Dallas
Semiconductors (now merged with Maxim). They relate to the same family
of chips. (iButton being a particular packaging method). This software
has not been reviewed or endorsed by Dallas, nor is the author associated
with them... other than as an appreciative customer! What I know about
MicroLan (a.k.a. 1-Wire) comes from studying generally available material
from Dallas. (Try www.ibutton.com, www.dalsemi.com and
www.arunet.co.uk/tkboyd/e1d.htm)

TMEX is Dallas's environment for MicroLan work. Program was developed
using TMEX 3.11 in a Win 3.11 environment. (The identical version
numbers are just coincidental.)


Also: copes poorly with absence of Dallas i/f to 1-wire.

Doesn't auto-detect either interface or chip IDs....
  but takes the IDs from an ini file.

*)


(*qrevise... Dallas DO provide a demo of reading a similar
device as part ofthe SDK. It is available in several flavours,
including a Delphi
version. It is a better program than this is... but you may find
this helps you grasp the ESSENTIALS without the distraction of
flexibility and robustness that any "for use" version of the
program should have.*)

(*qfix This demo doesn't show how to use some of the DS2423's features...*)


interface
(*This program started life as a Delphi 2 app. It was moved to Delphi 1,
  when I first replaced the Uses clause with one from a Delphi 1 App. It
  was then moved back to Delphi 2! There
  may be other issues, and there may be superfluous units.
  (iBTMEXPW was added, being needed to access TMEX)*)

  uses SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Menus, ExtCtrls, iBTMEXPW;
  (*NB: To Use iBTMEXPW, several strategies are possible. It may also be
  possible to avoid the need for this entry in the uses clause. One easy
  (if inelegant) way to enable using the unit is to copy Ibtmexpw.pas (which
  comes from Dallas as part of the SDK) into the project's folder.*)


const ver ='21 Sept 01';
     (*DS004 has complex history.
       Name assigned 21 Sept 01 (Day after Nik's wedding)
       I started with 1-Wire after seeing note about Tini
       in Computer Shopper, Nov 2000. www.computershopper.co.uk*)

type
TrTKFileOpener = record
     iErrMsg:integer;
     dfOut:textfile; (*Various entries in Delphi help confuse
                   issue of what to put here... text, textfile
                   file of text, etc. None seemed to be at the
                   bottom of....
                   I couldn't get this to work properly. Only
                   when I started using the global dfOut2 in
                   place of rFO.dfOut did writeln(<file handle>,sTmp)
                   start behaving. I COULD write short strings!*)
     end;

  TDS002f1 = class(TForm)
    buQuit: TButton;
    Memo1: TMemo;
    buHelp: TButton;
    paHelp: TPanel;
    meHelp: TMemo;
    buDoneWithHelp: TButton;
    meFile: TMemo;
    buGetCount: TButton;
    laCounterKludgeDisplay: TLabel;
    procedure buQuitClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure buHelpClick(Sender: TObject);
    procedure buDoneWithHelpClick(Sender: TObject);
    procedure buGetCountClick(Sender: TObject);

  private
    { Private declarations }
    rTmp:real;
    asTmp:string;(*IN DS001, this was ANSIString, allowing > 255 chars.
       All instances of asTmp fixed to less than 256 for DS002. (No
       ANSIString in Delphi1... is ther another way to get >255 chars
       in a memo line?*)
    cardTmp:cardinal;(*32bit unsigned*)
    sTmp,sTmp2:string;
    sChipID2423:string[16];
    iTmp:integer;
    iPortNum,iPortType,iResult,iFileHandle:smallint;
       (*N.B. Dallas TMEX (start of section 2.4 of SKD manual says that
       "short" in their terminology is a 16 bit signed integer. In
       Delphi 1 that is called an integer (shortint being a signed
       8-bit number. I had smallint in a Delphi 2 version of this, but
       a) Smallint doesn't occur in Delphi1, pp help file (even though
       program compiles okay), and b) I'm not sure I checked that
       Delphi 2 defined smallint as 16 bit.*)
    liTmp,liSHandle:longint;
    baStateBuffer:array[0..15361] of byte; (*15360 might do*)
    siaRom:array[0..8] of smallint;(*8.. not 7.. pp Dallas example...
        but things seen in Dallas doc suggests this (8) is a mistake.
        (7 would probably do just fine)*)
    baRomID2423:array[0..7] of byte;
    bTmp,bTmp2:byte;
    boShowMsgs:boolean;
    bitmap:TBitMap;
    rFO:TrTKFileOpener;
    dfIn:file of byte;
    boTmp,boNo1Wire:boolean;
    (*If boNo1Wire true, accesses to 1-wire suppressed*)
    procedure EstablishSession;
    procedure ChkSess(bWhich:byte);
    procedure DebugMessages(boOn:boolean;liTmp:longint);
    procedure DebugMessagesWLoc(boOn:boolean;liTmp:longint;bLoc:byte);
    procedure ShowTMErrMessage(bErrLoc:byte;liErr:longint;sErr:string);
    function GetCounterValue:cardinal; (*32bit unsigned*)

  public
    { Public declarations }
  end;

var
  DS002f1: TDS002f1;

implementation

{$R *.DFM}
{$R+} (*Turn on range checking. Unrelated to previous line
        which IS also needed.*)

procedure TDS002f1.ShowTMErrMessage(bErrLoc:byte;liErr:longint;sErr:string);
(*this is an unfinished bit of "cleverness"... but works!*)
begin
DebugMessagesWLoc(boShowMsgs,liErr,bErrLoc);
(*qFIX Replace showmessage with something like printer error retry*)
showmessage(sERR+' '+inttostr(liErr)+' at location'+inttostr(bErrLoc));
end;

procedure TDS002f1.buQuitClick(Sender: TObject);
begin
application.terminate;
end;

procedure TDS002f1.FormCreate(Sender: TObject);
var w1:word;
    c1:byte;

procedure FillMemos;
begin
with memo1 do begin
clear;
asTmp:='This program (c) and written by TK Boyd, 6/01. ';
lines.add(asTmp);
asTmp:='Rights reserved, except: Please feel free to pass it on, so long as you '+
'credit the source and mention that the author can be found on the net '+
'as follows:';
lines.add(asTmp);
asTmp:='www.arunet.co.uk/tkboyd/index2.htm should be valid. If not, '+
'search for Sheepdog Software.';
lines.add(asTmp);
lines.add(' ');
asTmp:='Now... how to keep conscience and lawyers happy? As novelists say, '+
'"I''d like to thank ... for the bits that are right, but all errors are mine." ';
lines.add(asTmp);
asTmp:='Writing this software was only possible through studying the excellent '+
'examples provided by Dallas Semiconductor. That is covered by typical fierce copyright '+
'notices and licenses, so I hope that no one there is going to complain that ';
lines.add(asTmp);
asTmp:='this breaches anything. Their software certainly does these things better '+
'than mine does, but I hope mine helps you get started.';
lines.add(asTmp);
asTmp:='The name "1-Wire" is a trademark of Dallas '+
'Semiconductor. Visit them at www.dalsemi.com. The author of '+
'this program is not connected with them, nor is the program endorsed '+
'by them.';
lines.add(asTmp);
end; (*With memo1*)

with meHelp do begin
clear;
asTmp:='''MicroLan'', etc, are TM of Dallas Semiconductor. '+
'See www.dalsemi.com and www.arunet.co.uk/tkboyd/e1d.htm';
lines.add(asTmp);
lines.add(' ');
asTmp:=''+
''+
'';
lines.add(asTmp);
lines.add(' ');
asTmp:=''+
''+
'';
lines.add(asTmp);
lines.add(' ');
end; (*With memo2*)
end;(*FillMenoms*)

procedure CheckIniFile;
var c1:byte;
begin
boTmp:=false;(*If true later, DS004.ini exisited and data in it was valid*)
if fileexists('DS004.ini') then begin
  boTmp:=true;
  with meFile do begin
  lines.loadfromfile('DS004.ini');
  if lines[0]<>'aaDS004.ini file' then begin(*3*)
    showmessage('1st line of ini file must be "aaDS004.ini file". '+
    'Program will terminate');
    application.terminate;
    boTmp:=false;(*in case app doesn't terminate...*)
    end;(*3*)

(*Dallas demos (in Delphi) show more elegant and robust versions
of the following initialisation code...
iPortNum is sets which COM port to look for adapter on.
iPortNum is sets to Adapter Type code (Dallas's code: 1 for DS9097E,
    5 for 9097-U, see Dallas docs for others)
Dallas recommend a call to TMGetTypeVersion here to verify port
type. There are other things you would also do in a more robust program
to ensure all was well with the system before attempting to use it.*)


  sTmp:=copy(lines[1],1,1);
  if pos(sTmp,'0123456789')=0 then begin(*3*)
     showmessage('1st char of second line sets the port the adapter is on. '+
     'It doesn''t seem to be a digit. Application will terminate.');
     application.terminate;(*Can anyone tell me why this didn't prevent
     and attempt at iPortNum:=strtoint(sTmp); (two lines down) when
     sTmp was equal to 'x'? The message appeared, as expected.*)
     boTmp:=false;(*in case app doesn't terminate...*)
     end;(*3*)
     iPortNum:=strtoint(sTmp);

  sTmp:=copy(lines[2],1,1);
  if pos(sTmp,'0123456789')=0 then begin(*3*)
     showmessage('1st char of third line sets the adapter type code. '+
     'It doesn''t seem to be a digit. Application will terminate.');
     application.terminate;(*Can anyone tell me why this didn't prevent
     and attempt at iPortNum:=strtoint(sTmp); (two lines down) when
     sTmp was equal to 'x'? The message appeared, as expected.*)
     boTmp:=false;(*in case app doesn't terminate...*)
     end;(*3*)
     iPortType:=strtoint(sTmp);

  sTmp:=copy(lines[3],1,4);
  if uppercase(sTmp)<>'IDC:' then begin(*3*)
    showmessage('Fourth line of ini file must start "IDC:". '+
    'Program will terminate');
    application.terminate;
    boTmp:=false;(*in case app doesn't terminate...*)
    end;(*3*)

  if length(lines[3])<>20 then begin(*3*)
    showmessage('Fourth line of ini file must be 20 characters long. '+
    'Program will terminate');
    application.terminate;
    boTmp:=false;(*in case app doesn't terminate...*)
    end;(*3*)

  sChipID2423:=uppercase(copy(lines[3],5,16));
  for c1:=1 to 16 do
    if pos(copy(sChipID2423,c1,1),'0123456789ABCDEF')=0 then begin(*3*)
       showmessage('There is an invalid character in the chip '+
       'ID. (Only 0-9, a-f, A-F allowed.) Application will terminate.');
       application.terminate;
       boTmp:=false;(*in case app doesn't terminate...*)
       end;(*3*)

   if copy(sChipID2423,15,2)<>'1D' then begin(*3*)
    showmessage('The id you gave for the counter is not for a DS2423. (LSB would '+
    'be 1D for that device.) Program could be expanded to work with '+
    'other devices, but has not been yet. Program will terminate');
    application.terminate;
    boTmp:=false;(*in case app doesn't terminate...*)
    end;(*3*)

  end;(*with meFile*)

  end; (*...of processing of DS004.ini*)

  if boTmp=false then begin
  showmessage('Unlike most programs from Sheepdog Software, '+
  'an .ini file is needed. It must be called DS004.ini and be '+
  'in the same folder as DS004.exe');
  showmessage('The ini file must consist of the following....'+#13+#13+
  'aaDS004.ini file'+#13+
  '2 <-1st char determines port where adapter is.'+#13+
  '5 <-1st char determines adapter type (see next msg)'+#13+
  'IDC:<followed by 16 digit id of DS2423, counter>');
  (*Possible .ini file follows.
  First 2 characters indicate the ini file format version.
  The second line determines whether the program
       writes temperature data to a log file.
  ID WILL need changing. The easy way to find your chip's id
       is to use the Dallas supplied iViewer.
=====================================================
aaDS004.ini file
2 <-1st char determines port where adapter is.
5 <-1st char determines adapter type (see next msg)
IDC:7E0008001DDED21D
=====================================================
*)


  showmessage('For adapter type in ini file:'+#13+
  '     5 for DS9097U'+#13+
  '     5 for LinkUSB from iButtonLink'+#13+
  '     1 for DS9097E'+#13+#13+
  'Program will now terminate.');
  application.terminate;
  end;(*if boTmp=false...no DS004.ini found, or it was faulty*)
if boTmp=false then application.terminate;(*Should have terminated
   before this if boTmp=false, but sometimes fails to so so in
   similar situations. "Belt and braces" code.*)
end;(*CheckIniFile*)

begin (*FormCreate*)

meFile.hide;(*Only a receptacle for contents of DS004.ini*)
meFile.Wordwrap:=false;(*Necessary when using a memo to hold ini files, etc*)
boShowMsgs:=not true;(*When false: suppresses debugging messages*)
paHelp.hide;
boNo1Wire:=false;

application.title:='DS004';
caption:='DS004- Read Counter  Ver: '+ver;

CheckIniFile;

baRomID2423[7]:=strtoint('$'+copy(sChipID2423,1,2)); (*MSB of chip's unique id.*)
baRomID2423[6]:=strtoint('$'+copy(sChipID2423,3,2));
baRomID2423[5]:=strtoint('$'+copy(sChipID2423,5,2));
baRomID2423[4]:=strtoint('$'+copy(sChipID2423,7,2));
baRomID2423[3]:=strtoint('$'+copy(sChipID2423,9,2));
baRomID2423[2]:=strtoint('$'+copy(sChipID2423,11,2));
baRomID2423[1]:=strtoint('$'+copy(sChipID2423,13,2));
baRomID2423[0]:=strtoint('$'+copy(sChipID2423,15,2));

(*In typical Delphi work, you would set these initial values with
the IDE, but I wanted such things in the source code because this
is written to demonstrate using the MicroLan devices*)


FillMemos;

end; {OnFormCreate}

procedure TDS002f1.DebugMessages(boOn:boolean;liTmp:longint);
   begin
   if boOn then showmessage('A TMEX call returned '+inttostr(liTmp));
   end;

procedure TDS002f1.DebugMessagesWLoc(boOn:boolean;liTmp:longint;bLoc:byte);
   begin
   if boOn then showmessage('A TMEX call returned '+inttostr(liTmp)+
     ' at location '+inttostr(bLoc));
   end;

procedure TDS002f1.EstablishSession;
  begin
  (*This next "improved" by TKB from code I thought was derived from
    Dallas....*)
   if liSHandle>0 then begin(*program believes a session established.
         if <1 then not appropriate to call TMEndSession*)
       iResult:=TMEndSession(liSHandle);
       if iResult<0 then (*if 0:had been relinq, 1:WAS valid, now relinq*)
          (*qfix... all such should not be showmessage because it is modal.
           should also probably incorp delay-before-proceed*)
              ShowTMErrMessage(1,iResult,'TMEndSession reported problem: ');
       end;(*if liSHandle>0*)
   liSHandle:=TMExtendedStartSession(iPortNum,iPortType,nil);(*Should return >0*)
   (*qfix... all old showmessages of this type should be changed to
     ShowTMErrMessage, not showmessage because it is modal.
     should also probably incorp delay-before-proceed*)
   if liSHandle<1 then ShowTMErrMessage(4,liSHandle,
                'TMExtendedStartSession reported problem: ');
   iResult:=TMSetUp(liSHandle);
   if iResult<>1 then ShowTMErrMessage(2,iResult,'TMSetUp reported problem: ');
  end;(*EstablishSession*)

procedure TDS002f1.ChkSess(bWhich:byte);
  (*Should not be called before there is an at-least-once-valid
    number in liSHandle.
    Check session still valid, restore if necessary*)
  begin
   iResult:=TMValidSession(liSHandle);
   if iResult<>1 then begin
   (*qfix... all such should not be showmessage because it is modal.
     should also probably incorp delay-before-proceed*)
     showmessage('Something made session invalid at check '+
        inttostr(bWhich)+'. Problem was: '+
        inttostr(iResult)+'. Re-establishing.');
     EstablishSession;
     end;
   end;

procedure TDS002f1.buHelpClick(Sender: TObject);
begin
paHelp.show;
paHelp.align:=alclient;
meHelp.align:=alclient;
buDoneWithHelp.top:=meHelp.height-25;
buDoneWithHelp.left:=meHelp.width-65;
end;

procedure TDS002f1.buDoneWithHelpClick(Sender: TObject);
begin
paHelp.hide;
end;

procedure TDS002f1.buGetCountClick(Sender: TObject);
begin
laCounterKludgeDisplay.caption:='Wait...';
application.processmessages;(*Without this, "wait.." didn't display*)
cardTmp:=GetCounterValue;
laCounterKludgeDisplay.caption:=inttostr(cardTmp);
if cardTmp=0 then showmessage('Be sure you can read the counter with '+
'the Dallas iButton Viewer. When that works, your problem may be that '+
'you have not edited DS004.ini to hold the unique ID of YOUR '+
'DS2423. Viewer will tell you the ID.');
end;

function TDS002f1.GetCounterValue:cardinal;

var c1:byte;

begin (*GetCounterValue*)
liSHandle:=TMExtendedStartSession(iPortNum,iPortType,nil);(*Should return >0*)
if liSHandle<1 then
   ShowTMErrMessage(113,liSHandle,'TMExtendedStartSession once at 835 reported problem: ');

iResult:=TMSetUp(liSHandle);
if iResult<>1 then ShowTMErrMessage(115,iResult,'TMSetUp once at 842 reported problem: ');

(*qfix.. edit*)
(*REVISE TO REFLECT GRASP OF SETUP/RESET DISTINCTION>>>>>Dallas docs say
this resets all devices on net. Also say that it should
be called once at start of APPLICATION... but suggest that needs calling
between TMExtendedStartSession and any other TM functions. The code here
works... but maybe a TMExtendedStartSession and ONE TMSetUp in
OnFormCreate, with subsequent TMExtendedStartSessions not having TMSetUp
would be right & entail less overhead? (I think that just one
TMExtendedStartSession is not sufficient for the whole app's run.)*)

for c1:=0 to 7 do siaROM[c1]:=baRomID2423[c1];
(*repeatedly refilling may not be necessary. If the previous
line is right, some Dallas docs are less than fully clear (to me)
on some important matters regarding the last parameter of TMRom

My grasp of using TMEX is a work in progress. An example...
We know that in the code we write we can set up arrays, e.g. baRomID.
We know that there are groups of bytes in the hardware, e.g. the DS2405,
   which we want to know about.
Less obvious is a scratchpad which I think exists within the driver
   between our program and the hardware.
I THINK the essential point is that TMRom doesn't directly read/write the
    hardware. Rather, it reads/writes the scratchpad. Other routines
    cause things to go between the scratchpad and the hardware in due season.
If the first element of the last parameter of TMRom is 0, TMRom will read
    from the scratchpad. The implication of this is that no device would
    have a device ID with 0 in this byte... otherwise there would be no
    way to put the relevant device id in the scratchpad prior to an
    "access <that> device" call... but this seems a reasonable constraint
    that the MicroLan designers may have accepted.
*)

iResult:=TMRom(liSHandle,@baStateBuffer,@siaROM); (*Load buffer in
  s/w driver with ID of chip to be accessed.*)
  (*if iResult=1 after call, transfer succeeded*)
if iResult<>1 then
     ShowTMErrMessage(116,iResult,'TMRom reported problem: ');

ChkSess(41);(*Shouldn't be necessary, but.... Check session and
   re-establish if necessary.*)

(*This based on Read DS1994 Delphi example supplied with SDK.
Seems to work... not too sure how robust error handling is.*)
cardTmp:=0;
if liSHandle>0 then begin (*2*)
 bTmp:=0;
 repeat
   if TMStrongAccess(liSHandle,@baStateBuffer)=1
     then begin (*1*)
       TMTouchByte(liSHandle,$A5); (*Read Memory+Counter command*)
//BE SURE $E0/$F0, 14/15, A/B all right

//C0/14/A seems right, pp dallas datasheet

(*Next, two more bytes sent to the TMEX drivers....

Doing the following, at this point in the dialog, prepares the system
   to read information from the memory page starting at 01C0h (aka $01C0)...

       TMTouchByte(liSHandle,$C0); //Send LSB of 16 bit address
       TMTouchByte(liSHandle,$01); //Send MSB of 16 bit address
*)

//THERE'S A FLAW IN THE PROGRAM... if counter is at ZERO, prgm throws up
   //message suggesting a fault has happened.

      // TMTouchByte(liSHandle,$C0); (*Point to page 14 to read input A counter*)
       TMTouchByte(liSHandle,$E0); (*Point to page 15 to read input B counter*)

       TMTouchByte(liSHandle,$01); (*... second byte to spec page*)

       for c1:=0 to 31 (*Discard bytes reporting memory contents*)
         do bTmp2:=TMTouchByte(liSHandle,$FF);

(*Read and accumulate the four bytes of the counter...
    They are delivered LSB first*)

       cardTmp:=TMTouchByte(liSHandle,$FF);
       bTmp2:=TMTouchByte(liSHandle,$FF);
         cardTmp:=bTmp2*256+cardTmp;
       bTmp2:=TMTouchByte(liSHandle,$FF);
         cardTmp:=bTmp2*65536+cardTmp;(*65536=256*256  *)
       bTmp2:=TMTouchByte(liSHandle,$FF);
         cardTmp:=bTmp2*16777216+cardTmp;(*16777216=256*256*256  *)

       (*You should also, next, do something like...

       for c1:=0 to 3 do begin
       bTmp1:=TMTouchByte(liSHandle,$FF);
       end; skip the 4 bytes of zeros expected after the 4 bytes
              of counter info

       CS1:=TMTouchByte(liSHandle,$FF);
       CS2:=TMTouchByte(liSHandle,$FF);
       (and combine them into a 16 bit number)

       ... to here read th two bytes of the (inverted) 16 bit CRC checksum.
       And see that the data and the checksum are consistent with
       one another.

       But you don't have to. Whew.
       *)

   end;(*1*)
   inc(bTmp);
until (bTmp>4) or (cardTmp<>0);
   (*Looks like a 'try 4 times if MicroLan playing up' kludge...
      and will be subject to a rollover error once in 2^32 counts*)
end;(*2*)

iResult:=TMEndSession(liSHandle);
DebugMessages(boShowMsgs,iResult);
if iResult<0
 then ShowTMErrMessage(120,iResult,'TMEndSession xx reported problem: ');

result:=cardTmp;
end;(*GetCounterValue*)

end.
